(document:surround "/std/frame")
(document:insert "/std/functions")
(document:insert "/users/functions")

(document:envelop with-translation _ "alterator-users")

(document:envelop with-attributes
                  (field-text field-activity field-changed
                              password1 password2 password-changed))

(document:envelop with-container-presentations
                  ((field '/users/field field-text)
                   (password-box '/users/passwd)))

;;;;;;;;;;;;;;;;;

(frame:on-leave
   (lambda()
     (and (group2 visibility) 'need-confirm)))

(frame:on-apply
   (lambda()
     (case (cell-ref mode)
       ((new) (new-all))
       ((edit) (change-all)))))

;;;;;;;;;;;;;;;;;

(define mode (make-cell 'edit))

(define (users)
  (woo-catch
   (lambda() (users-list))
   (lambda(reason) '())))

(define (check-buttons)
  ((widgets do-delete do-edit)
   activity (> (userlist count) 0)))

(define (update-users)
  (let ((users (users)))
    (userlist rows users)
    (or (null? users)
        (begin (userlist current 0
                         selected))))
  #t)

;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (change-all)
  (woo-catch/message
   (thunk
     (users-write (userlist text)
                  'allow_su (allow_su state)
                  'gecos (gecos field-text)
                  'passwd1 (p-box password1)
                  'passwd2 (p-box password2))
     #t)))

(define (new-all)
  (woo-catch/message
   (thunk
    (users-new (username field-text)
               'allow_su (allow_su state)
               'gecos (gecos field-text)
               'passwd1 (p-box password1)
               'passwd2 (p-box password2))
    #t)))

(define (delete-all)
  (and (eq? 'yes (document:popup-warning
                 (string-append (_ "Do you really want to delete user ")
                                (userlist text)
                                " ?")
                 'no
                 'yes))
       (woo-catch/message
        (thunk
          (users-delete (userlist text))
          (update-users)))
       (check-buttons)))


;;;;;;;;;;;;;;;;;;;;;;

(define (fill-general-options)
  ((widgets username gecos) field-text "")
  (and (> (userlist count) 0)
       (eq? (cell-ref mode) 'edit)
       (let* ((name (userlist text))
              (data (users-read name)))
         (allow_su state (woo-get-option data 'allow_su #f))
         (gecos field-text (woo-get-option data 'gecos))
         (username field-text name))))

(define (fill-chpasswd)
  (p-box password1 ""
         password2 ""))

(define (fill-all)
  (fill-general-options)
  (fill-chpasswd))

(define (toggle-activity)
  (let ((state (userlist activity)))
    (group2 visibility state)
    (group1 visibility (not state))
    (check-buttons)
    
    (userlist activity (not state))
    (allow_su activity state)
    (field-toggle gecos state)
    (p-box visibility state)))

(define (go-new-mode)
  (update-constraints "new" "/users")
  (field-toggle username #t)
  (cell-set! mode 'new)

  (fill-all)

  (allow_su state #f)
  (userlist append-row (_ "NEW USER"))
  (userlist current (- (userlist count) 1)))

(define (go-edit-mode)
  (update-constraints "write" "/users")
  (cell-set! mode 'edit)
  (field-toggle username #f)
  #t)

;;;;;;;;;;;;;;;;;;;;;;
(define (field-toggle widget state)
  (widget field-activity state))

;;;;;;;;;;;;;;;;;;;;;;;;

(define (general-options-tab)
  (document:id general-options
               (tab (_ "General Options")
                    margin 10
                    (vbox spacing 2
                          margin 2
                          (hbox (label (bold (_ "Name:")))
                                (document:id username
                                             (field "<name of some user>"
                                                    widget-name "name"
                                                    (when field-changed
                                                      (userlist row-item
                                                                (userlist current)
                                                                (username field-text))))))
                          (hbox (label (bold (_ "Gecos:")))
                                (document:id gecos
                                             (field "<comment of some user>"
					      widget-name "gecos")))
                          (hbox (label (bold (_ "Allow switching to superuser")))
                                (document:id allow_su
                                             (checkbox "" inactive)))
                          (groupbox (_ "Password:")
			            max-height 140
                                    (document:id p-box (password-box invisible)))
			  (vertical-spacer)))))

(define (buttons)
  (list
   (document:id group1 (hbox
                        margin 10
                        (document:id do-edit
                                     (button (_ "Edit") inactive (when clicked (toggle-activity))))
                        (document:id do-new
                                     (button (_ "New") (when clicked (toggle-activity)
                                                             (go-new-mode))))
                        (document:id do-delete
                                     (button (_ "Delete") inactive (when clicked (delete-all))))))
   
   (document:id group2 (hbox  invisible
                              margin 10
                              (document:id apply-button
                                           (button (_ "Apply")
                                                   (when clicked
                                                     (and (case (cell-ref mode)
                                                            ((edit) (change-all))
                                                            ((new) (and (new-all)
                                                                        (go-edit-mode)
                                                                        (update-users))))
                                                          (begin (fill-all)
                                                                 (toggle-activity))))))
                              
                              (document:id discard-button
                                           (button (_ "Discard")
                                                   (when clicked
                                                     (and (eq? (cell-ref mode) 'new)
                                                          (begin (go-edit-mode)
                                                                 (userlist remove (userlist current))
                                                                 (and (> (userlist count) 0)
                                                                      (userlist current 0))))
                                                     (fill-all)
                                                     (toggle-activity))))))))

;;;;;;;;;;;;;;;;;;;;;;;

(hbox 
      (document:id userlist
                   (listbox rows (users)
                            (when selected
                              ((widgets do-edit do-delete)
                               activity #t)
                              (fill-all))))
      (vbox 
       (tabbox
        (general-options-tab))
       (buttons)))

(button (_ "Quit") (when clicked (document:end))) 
